==> hth-0.1.tar.gz: hth-0.1/rule.c <==
#include <stdio.h>

#include "rule.h"

#define FLAG_CAPTURED 128

static int capture_piece_clean(int *state);
static int capture_piece_hh(int *state, int x0, int y0,
                            int head_friendly, int dx, int dy);
static int capture_piece_tenbin(int *state, int x0, int y0,
                                int head_friendly, int dx, int dy);

int
state_reset(int *state)
{
  int i;

  if (state == NULL)
  {
    fprintf(stderr, "state_reset: state is NULL\n");
    return 1;
  }

  for (i = 0; i < 81; i++)
    state[18 + i] = -1;

  for (i = 0; i < 9; i++)
  {
    state[i] = i;
    state[18 + i] = i;

    state[9 + i] = 72 + i;
    state[18 + (72 + i)] = 9 + i;
  }

  state[99] = 9;
  state[100] = 9;

  return 0;
}

int
state_is_sane(int *state)
{
  int i;
  int num_blue;
  int num_red;

  if (state == NULL)
  {
    fprintf(stderr, "state_is_sane: state is NULL\n");
    return 0;
  }

  for (i = 0; i < 18; i++)
  {
    if (state[i] < 0)
      continue;
    if ((state[i] < 0) || (state[i] >= 81))
    {
      fprintf(stderr, "state_is_sane: piece %d is out of board (%d)\n",
              i, state[i]);
      return 0;
    }
    if (state[18 + state[i]] != i)
    {
      fprintf(stderr, "state_is_sane: piece %d mismatch "
              "(piece %d = %d, grid %d = %d)\n",
              i, i, state[i], state[i], state[18 + state[i]]);
      return 0;
    }
  }

  for (i = 0; i < 81; i++)
  {
    if (state[18 + i] < 0)
      continue;
    if (state[18 + i] >= 18)
    {
      fprintf(stderr, "state_is_sane: strange piece in grid %d (%d)\n",
              i, state[18 + 1]);
      return 0;
    }
    if (state[state[18 + i]] != i)
    {
      fprintf(stderr, "state_is_sane: grid %d mismatch "
              "(grid %d = %d, piece %d = %d)\n",
              i, i, state[18 + i], state[18 + i], state[state[18 + i]]);
      return 0;
    }
  }

  if (count_piece(state, &num_blue, &num_red) != 0)
  {
    fprintf(stderr, "state_is_sane: count_piece failed\n");
    return 0;
  }
  if (num_blue != state[99])
  {
    fprintf(stderr, "state_is_sane: num_blue mismatch "
            "(num_blue = %d, state[99] = %d)\n",
            num_blue, state[99]);
    return 0;
  }
  if (num_red != state[100])
  {
    fprintf(stderr, "state_is_sane: num_red mismatch "
            "(num_red = %d, state[100] = %d)\n",
            num_red, state[100]);
    return 0;
  }

  return 1;
}

int
move_is_legal(int *state, int turn_player, int grid_from, int grid_to,
              int victory_condition)
{
  int g;
  int delta;

  if (state == NULL)
  {
    fprintf(stderr, "move_is_legal: state is NULL\n");
    return 0;
  }

  if ((turn_player < 0) || (turn_player >= 2))
    return 0;

  if ((state[99] < victory_condition) || (state[100] < victory_condition))
    return 0;

  if ((grid_from < 0) || (grid_from >= 81))
    return 0;
  if ((state[18 + grid_from] < turn_player * 9 + 0)
      || (state[18 + grid_from] >= turn_player * 9 + 9))
    return 0;

  if ((grid_to < 0) || (grid_to >= 81))
    return 0;
  if (state[18 + grid_to] >= 0)
    return 0;

  delta = 0;
  if (grid_from % 9 == grid_to % 9)
  {
    if (grid_from / 9 < grid_to / 9)
      delta = 9;
    else
      delta = -9;
  }
  else if (grid_from / 9 == grid_to / 9)
  {
    if (grid_from % 9 < grid_to % 9)
      delta = 1;
    else
      delta = -1;
  }
  else
  {
    return 0;
  }

  for (g = grid_from + delta; g != grid_to; g += delta)
  {
    if ((g < 0) || (g >= 81))
    {
      fprintf(stderr, "move_is_legal: jump check went out of board\n");
      return 0;
    }

    if (state[18 + g] >= 0)
      return 0;
  }

  return 1;
}

/* this does _not_ capture pieces
 * this does _not_ check if the move is legal
 */
int
move_piece(int *state, int grid_from, int grid_to)
{
  if (state == NULL)
  {
    fprintf(stderr, "move_piece: state is NULL\n");
    return 1;
  }

  if ((grid_from < 0) || (grid_from >= 81))
    return 1;
  if ((grid_to < 0) || (grid_to >= 81))
    return 1;

  if ((state[18 + grid_from] < 0)
      || (state[18 + grid_from] >= 18))
    return 1;

  state[18 + grid_to] = state[18 + grid_from];
  state[state[18 + grid_from]] = grid_to;
  state[18 + grid_from] = -1;

  return 0;
}

static int
capture_piece_clean(int *state)
{
  int i;

  for (i = 0; i < 18; i++)
  {
    if (state[i] < 0)
      continue;

    if (state[i] & FLAG_CAPTURED)
    {
      state[18 + (state[i] & (~FLAG_CAPTURED))] = -2;
      state[i] = -1;

      if (i < 9)
        (state[99])--;
      else
        (state[100])--;
    }
  }

  return 0;
}

/* hasami and hashi-otoshi
 * return 1 if a piece is captured, 0 otherwise
 */
static int
capture_piece_hh(int *state, int x0, int y0,
                 int head_friendly, int dx, int dy)
{
  int i;
  int x;
  int y;
  int g;
  int capture_ok;
  int capture_count;

  x = x0 + dx;
  y = y0 + dy;
  capture_ok = 0;
  capture_count = 0;
  while ((x >= 0) && (x < 9) && (y >= 0) && (y < 9))
  {
    g = y * 9 + x;

    if (state[18 + g] < 0)
    {
      return 0;
    }
    else if ((state[18 + g] >= head_friendly + 0)
             && (state[18 + g] < head_friendly + 9))
    {
      capture_ok = 1;
      break;
    }
    else
    {
      capture_count++;
    }

    x += dx;
    y += dy;
    if (((x < 0) || (x >= 9)) && (y >= 0) && (y < 9))
    {
      capture_ok = 1;
      break;
    }
  }

  if (!capture_ok)
    return 0;
  if (capture_count < 1)
    return 0;

  for (i = 1; i <= capture_count; i++)
  {
    g = (y0 + dy * i) * 9 + (x0 + dx * i);
    state[state[18 + g]] |= FLAG_CAPTURED;
  }

  return 1;
}

/* return 1 if a piece is captured, 0 otherwise */
static int
capture_piece_tenbin(int *state, int x0, int y0,
                     int head_friendly, int dx, int dy)
{
  int x;
  int y;
  int g;
  int target_a;
  int target_b;

  target_a = -1;
  x = x0 + dx;
  y = y0 + dy;
  while ((x >= 0) && (x < 9) && (y >= 0) && (y < 9))
  {
    g = y * 9 + x;

    if (state[18 + g] < 0)
    {
      return 0;
    }
    else if ((state[18 + g] >= head_friendly + 0)
             && (state[18 + g] < head_friendly + 9))
    {
      /* do nothing */;
    }
    else
    {
      target_a = g;
      break;
    }

    x += dx;
    y += dy;
  }
  if (target_a < 0)
    return 0;

  target_b = -1;
  x = x0 - dx;
  y = y0 - dy;
  while ((x >= 0) && (x < 9) && (y >= 0) && (y < 9))
  {
    g = y * 9 + x;

    if (state[18 + g] < 0)
    {
      return 0;
    }
    else if ((state[18 + g] >= head_friendly + 0)
             && (state[18 + g] < head_friendly + 9))
    {
      /* do nothing */;
    }
    else
    {
      target_b = g;
      break;
    }

    x -= dx;
    y -= dy;
  }
  if (target_b < 0)
    return 0;

  state[state[18 + target_a]] |= FLAG_CAPTURED;
  state[state[18 + target_b]] |= FLAG_CAPTURED;

  return 1;
}

/* optimized under the assumption that a piece is usually not
 * adjacent to other pieces
 */
int
capture_piece(int *state, int grid_to)
{
  int x0;
  int y0;
  int head_friendly;
  int adjacent_piece_a;
  int adjacent_piece_b;
  int captured;

  if (state == NULL)
  {
    fprintf(stderr, "capture_piece: state is NULL\n");
    return 1;
  }
  if ((grid_to < 0) || (grid_to >= 81))
  {
    fprintf(stderr, "capture_piece: grid_to is out of board (%d)\n",
            grid_to);
    return 1;
  }
  if (state[18 + grid_to] < 0)
  {
    fprintf(stderr, "capture_piece: no piece in grid %d\n",
            grid_to);
    return 1;
  }
  if (state[18 + grid_to] >= 18)
  {
    fprintf(stderr, "capture_piece: strange piece in grid %d (%d)\n",
            grid_to, state[18 + grid_to]);
    return 1;
  }

  x0 = grid_to % 9;
  y0 = grid_to / 9;
  if (state[18 + grid_to] < 9)
    head_friendly = 0;
  else
    head_friendly = 9;

  captured = 0;

  adjacent_piece_a = 0;
  adjacent_piece_b = 0;
  if ((x0 >= 1) && (state[18 + (grid_to - 1)] >= 0))
  {
    adjacent_piece_a = 1;
    captured |= capture_piece_hh(state, x0, y0, head_friendly, -1, 0);
  }
  if ((x0 < 8) && (state[18 + (grid_to + 1)] >= 0))
  {
    adjacent_piece_b = 1;
    captured |= capture_piece_hh(state, x0, y0, head_friendly, 1, 0);
  }
  if ((adjacent_piece_a) && (adjacent_piece_b))
    captured |= capture_piece_tenbin(state, x0, y0, head_friendly, 1, 0);

  adjacent_piece_a = 0;
  adjacent_piece_b = 0;
  if ((x0 < 8) && (y0 >= 1) && (state[18 + (grid_to - 8)] >= 0))
  {
    adjacent_piece_a = 1;
    captured |= capture_piece_hh(state, x0, y0, head_friendly, 1, -1);
  }
  if ((x0 >= 1) && (y0 < 8) && (state[18 + (grid_to + 8)] >= 0))
  {
    adjacent_piece_b = 1;
    captured |= capture_piece_hh(state, x0, y0, head_friendly, -1, 1);
  }
  if ((adjacent_piece_a) && (adjacent_piece_b))
    captured |= capture_piece_tenbin(state, x0, y0, head_friendly, 1, -1);

  adjacent_piece_a = 0;
  adjacent_piece_b = 0;
  if ((y0 >= 1) && (state[18 + (grid_to - 9)] >= 0))
  {
    adjacent_piece_a = 1;
    captured |= capture_piece_hh(state, x0, y0, head_friendly, 0, -1);
  }
  if ((y0 < 8) && (state[18 + (grid_to + 9)] >= 0))
  {
    adjacent_piece_b = 1;
    captured |= capture_piece_hh(state, x0, y0, head_friendly, 0, 1);
  }
  if ((adjacent_piece_a) && (adjacent_piece_b))
    captured |= capture_piece_tenbin(state, x0, y0, head_friendly, 0, 1);

  adjacent_piece_a = 0;
  adjacent_piece_b = 0;
  if ((x0 >= 1) && (y0 >= 1) && (state[18 + (grid_to - 10)] >= 0))
  {
    adjacent_piece_a = 1;
    captured |= capture_piece_hh(state, x0, y0, head_friendly, -1, -1);
  }
  if ((x0 < 8) && (y0 < 8) && (state[18 + (grid_to + 10)] >= 0))
  {
    adjacent_piece_b = 1;
    captured |= capture_piece_hh(state, x0, y0, head_friendly, 1, 1);
  }
  if ((adjacent_piece_a) && (adjacent_piece_b))
    captured |= capture_piece_tenbin(state, x0, y0, head_friendly, 1, 1);

  if (captured)
    capture_piece_clean(state);

  return 0;
}

int
remove_corpse(int *state)
{
  int i;

  if (state == NULL)
  {
    fprintf(stderr, "remove_corpse: state is NULL\n");
    return 1;
  }

  for (i = 0; i < 81; i++)
  {
    if (state[18 + i] == -2)
      state[18 + i] = -1;
  }

  return 0;
}

int
count_piece(int *state, int *num_blue, int *num_red)
{
  int i;

  if (state == NULL)
  {
    fprintf(stderr, "count_piece: state is NULL\n");
    return 1;
  }

  if (num_blue != NULL)
  {
    (*num_blue) = 0;
    for (i = 0; i < 9; i++)
    {
      if (state[i] < 0)
        continue;

      (*num_blue)++;
    }
  }

  if (num_red != NULL)
  {
    (*num_red) = 0;
    for (i = 9; i < 18; i++)
    {
      if (state[i] < 0)
        continue;

      (*num_red)++;
    }
  }

  return 0;
}

==> hth-0.2.tar.gz: hth-0.2/rule.c <==
#include <stdio.h>

#include "rule.h"

#define FLAG_CAPTURED 128

static int capture_piece_clean(int *state);
static int capture_piece_hh(int *state, int x0, int y0,
                            int head_friendly, int dx, int dy);
static int capture_piece_tenbin(int *state, int x0, int y0,
                                int head_friendly, int dx, int dy);

int
state_reset(int *state)
{
  int i;

  if (state == NULL)
  {
    fprintf(stderr, "state_reset: state is NULL\n");
    return 1;
  }

  for (i = 0; i < 81; i++)
    state[18 + i] = -1;

  for (i = 0; i < 9; i++)
  {
    state[i] = i;
    state[18 + i] = i;

    state[9 + i] = 72 + i;
    state[18 + (72 + i)] = 9 + i;
  }

  state[99] = 9;
  state[100] = 9;

  return 0;
}

int
state_is_sane(int *state)
{
  int i;
  int num_blue;
  int num_red;

  if (state == NULL)
  {
    fprintf(stderr, "state_is_sane: state is NULL\n");
    return 0;
  }

  for (i = 0; i < 18; i++)
  {
    if (state[i] < 0)
      continue;
    if ((state[i] < 0) || (state[i] >= 81))
    {
      fprintf(stderr, "state_is_sane: piece %d is out of board (%d)\n",
              i, state[i]);
      return 0;
    }
    if (state[18 + state[i]] != i)
    {
      fprintf(stderr, "state_is_sane: piece %d mismatch "
              "(piece %d = %d, grid %d = %d)\n",
              i, i, state[i], state[i], state[18 + state[i]]);
      return 0;
    }
  }

  for (i = 0; i < 81; i++)
  {
    if (state[18 + i] < 0)
      continue;
    if (state[18 + i] >= 18)
    {
      fprintf(stderr, "state_is_sane: strange piece in grid %d (%d)\n",
              i, state[18 + 1]);
      return 0;
    }
    if (state[state[18 + i]] != i)
    {
      fprintf(stderr, "state_is_sane: grid %d mismatch "
              "(grid %d = %d, piece %d = %d)\n",
              i, i, state[18 + i], state[18 + i], state[state[18 + i]]);
      return 0;
    }
  }

  if (count_piece(state, &num_blue, &num_red) != 0)
  {
    fprintf(stderr, "state_is_sane: count_piece failed\n");
    return 0;
  }
  if (num_blue != state[99])
  {
    fprintf(stderr, "state_is_sane: num_blue mismatch "
            "(num_blue = %d, state[99] = %d)\n",
            num_blue, state[99]);
    return 0;
  }
  if (num_red != state[100])
  {
    fprintf(stderr, "state_is_sane: num_red mismatch "
            "(num_red = %d, state[100] = %d)\n",
            num_red, state[100]);
    return 0;
  }

  return 1;
}

int
move_is_legal(int *state, int turn_player, int grid_from, int grid_to,
              int victory_condition)
{
  int g;
  int delta;

  if (state == NULL)
  {
    fprintf(stderr, "move_is_legal: state is NULL\n");
    return 0;
  }

  if ((turn_player < 0) || (turn_player >= 2))
    return 0;

  if ((state[99] < victory_condition) || (state[100] < victory_condition))
    return 0;

  if ((grid_from < 0) || (grid_from >= 81))
    return 0;
  if ((state[18 + grid_from] < turn_player * 9 + 0)
      || (state[18 + grid_from] >= turn_player * 9 + 9))
    return 0;

  if ((grid_to < 0) || (grid_to >= 81))
    return 0;
  if (state[18 + grid_to] >= 0)
    return 0;

  delta = 0;
  if (grid_from % 9 == grid_to % 9)
  {
    if (grid_from / 9 < grid_to / 9)
      delta = 9;
    else
      delta = -9;
  }
  else if (grid_from / 9 == grid_to / 9)
  {
    if (grid_from % 9 < grid_to % 9)
      delta = 1;
    else
      delta = -1;
  }
  else
  {
    return 0;
  }

  for (g = grid_from + delta; g != grid_to; g += delta)
  {
    if ((g < 0) || (g >= 81))
    {
      fprintf(stderr, "move_is_legal: jump check went out of board\n");
      return 0;
    }

    if (state[18 + g] >= 0)
      return 0;
  }

  return 1;
}

/* this does _not_ capture pieces
 * this does _not_ check if the move is legal
 */
int
move_piece(int *state, int grid_from, int grid_to)
{
  if (state == NULL)
  {
    fprintf(stderr, "move_piece: state is NULL\n");
    return 1;
  }

  if ((grid_from < 0) || (grid_from >= 81))
    return 1;
  if ((grid_to < 0) || (grid_to >= 81))
    return 1;

  if ((state[18 + grid_from] < 0)
      || (state[18 + grid_from] >= 18))
    return 1;

  state[18 + grid_to] = state[18 + grid_from];
  state[state[18 + grid_from]] = grid_to;
  state[18 + grid_from] = -1;

  return 0;
}

static int
capture_piece_clean(int *state)
{
  int i;

  for (i = 0; i < 18; i++)
  {
    if (state[i] < 0)
      continue;

    if (state[i] & FLAG_CAPTURED)
    {
      state[18 + (state[i] & (~FLAG_CAPTURED))] = -2;
      state[i] = -1;

      if (i < 9)
        (state[99])--;
      else
        (state[100])--;
    }
  }

  return 0;
}

/* hasami and hashi-otoshi
 * return 1 if a piece is captured, 0 otherwise
 */
static int
capture_piece_hh(int *state, int x0, int y0,
                 int head_friendly, int dx, int dy)
{
  int i;
  int x;
  int y;
  int g;
  int capture_ok;
  int capture_count;

  x = x0 + dx;
  y = y0 + dy;
  capture_ok = 0;
  capture_count = 0;
  while ((x >= 0) && (x < 9) && (y >= 0) && (y < 9))
  {
    g = y * 9 + x;

    if (state[18 + g] < 0)
    {
      return 0;
    }
    else if ((state[18 + g] >= head_friendly + 0)
             && (state[18 + g] < head_friendly + 9))
    {
      capture_ok = 1;
      break;
    }
    else
    {
      capture_count++;
    }

    x += dx;
    y += dy;
    /* hashi-otoshi */
    if (((x < 0) || (x >= 9)) && (dy == 0))
    {
      capture_ok = 1;
      break;
    }
  }

  if (!capture_ok)
    return 0;
  if (capture_count < 1)
    return 0;

  for (i = 1; i <= capture_count; i++)
  {
    g = (y0 + dy * i) * 9 + (x0 + dx * i);
    state[state[18 + g]] |= FLAG_CAPTURED;
  }

  return 1;
}

/* return 1 if a piece is captured, 0 otherwise */
static int
capture_piece_tenbin(int *state, int x0, int y0,
                     int head_friendly, int dx, int dy)
{
  int x;
  int y;
  int g;
  int target_a;
  int target_b;

  target_a = -1;
  x = x0 + dx;
  y = y0 + dy;
  while ((x >= 0) && (x < 9) && (y >= 0) && (y < 9))
  {
    g = y * 9 + x;

    if (state[18 + g] < 0)
    {
      return 0;
    }
    else if ((state[18 + g] >= head_friendly + 0)
             && (state[18 + g] < head_friendly + 9))
    {
      /* do nothing */;
    }
    else
    {
      target_a = g;
      break;
    }

    x += dx;
    y += dy;
  }
  if (target_a < 0)
    return 0;

  target_b = -1;
  x = x0 - dx;
  y = y0 - dy;
  while ((x >= 0) && (x < 9) && (y >= 0) && (y < 9))
  {
    g = y * 9 + x;

    if (state[18 + g] < 0)
    {
      return 0;
    }
    else if ((state[18 + g] >= head_friendly + 0)
             && (state[18 + g] < head_friendly + 9))
    {
      /* do nothing */;
    }
    else
    {
      target_b = g;
      break;
    }

    x -= dx;
    y -= dy;
  }
  if (target_b < 0)
    return 0;

  state[state[18 + target_a]] |= FLAG_CAPTURED;
  state[state[18 + target_b]] |= FLAG_CAPTURED;

  return 1;
}

/* optimized under the assumption that a piece is usually not
 * adjacent to other pieces
 */
int
capture_piece(int *state, int grid_to)
{
  int x0;
  int y0;
  int head_friendly;
  int adjacent_piece_a;
  int adjacent_piece_b;
  int captured;

  if (state == NULL)
  {
    fprintf(stderr, "capture_piece: state is NULL\n");
    return 1;
  }
  if ((grid_to < 0) || (grid_to >= 81))
  {
    fprintf(stderr, "capture_piece: grid_to is out of board (%d)\n",
            grid_to);
    return 1;
  }
  if (state[18 + grid_to] < 0)
  {
    fprintf(stderr, "capture_piece: no piece in grid %d\n",
            grid_to);
    return 1;
  }
  if (state[18 + grid_to] >= 18)
  {
    fprintf(stderr, "capture_piece: strange piece in grid %d (%d)\n",
            grid_to, state[18 + grid_to]);
    return 1;
  }

  x0 = grid_to % 9;
  y0 = grid_to / 9;
  if (state[18 + grid_to] < 9)
    head_friendly = 0;
  else
    head_friendly = 9;

  captured = 0;

  adjacent_piece_a = 0;
  adjacent_piece_b = 0;
  if ((x0 >= 1) && (state[18 + (grid_to - 1)] >= 0))
  {
    adjacent_piece_a = 1;
    captured |= capture_piece_hh(state, x0, y0, head_friendly, -1, 0);
  }
  if ((x0 < 8) && (state[18 + (grid_to + 1)] >= 0))
  {
    adjacent_piece_b = 1;
    captured |= capture_piece_hh(state, x0, y0, head_friendly, 1, 0);
  }
  if ((adjacent_piece_a) && (adjacent_piece_b))
    captured |= capture_piece_tenbin(state, x0, y0, head_friendly, 1, 0);

  adjacent_piece_a = 0;
  adjacent_piece_b = 0;
  if ((x0 < 8) && (y0 >= 1) && (state[18 + (grid_to - 8)] >= 0))
  {
    adjacent_piece_a = 1;
    captured |= capture_piece_hh(state, x0, y0, head_friendly, 1, -1);
  }
  if ((x0 >= 1) && (y0 < 8) && (state[18 + (grid_to + 8)] >= 0))
  {
    adjacent_piece_b = 1;
    captured |= capture_piece_hh(state, x0, y0, head_friendly, -1, 1);
  }
  if ((adjacent_piece_a) && (adjacent_piece_b))
    captured |= capture_piece_tenbin(state, x0, y0, head_friendly, 1, -1);

  adjacent_piece_a = 0;
  adjacent_piece_b = 0;
  if ((y0 >= 1) && (state[18 + (grid_to - 9)] >= 0))
  {
    adjacent_piece_a = 1;
    captured |= capture_piece_hh(state, x0, y0, head_friendly, 0, -1);
  }
  if ((y0 < 8) && (state[18 + (grid_to + 9)] >= 0))
  {
    adjacent_piece_b = 1;
    captured |= capture_piece_hh(state, x0, y0, head_friendly, 0, 1);
  }
  if ((adjacent_piece_a) && (adjacent_piece_b))
    captured |= capture_piece_tenbin(state, x0, y0, head_friendly, 0, 1);

  adjacent_piece_a = 0;
  adjacent_piece_b = 0;
  if ((x0 >= 1) && (y0 >= 1) && (state[18 + (grid_to - 10)] >= 0))
  {
    adjacent_piece_a = 1;
    captured |= capture_piece_hh(state, x0, y0, head_friendly, -1, -1);
  }
  if ((x0 < 8) && (y0 < 8) && (state[18 + (grid_to + 10)] >= 0))
  {
    adjacent_piece_b = 1;
    captured |= capture_piece_hh(state, x0, y0, head_friendly, 1, 1);
  }
  if ((adjacent_piece_a) && (adjacent_piece_b))
    captured |= capture_piece_tenbin(state, x0, y0, head_friendly, 1, 1);

  if (captured)
    capture_piece_clean(state);

  return 0;
}

int
remove_corpse(int *state)
{
  int i;

  if (state == NULL)
  {
    fprintf(stderr, "remove_corpse: state is NULL\n");
    return 1;
  }

  for (i = 0; i < 81; i++)
  {
    if (state[18 + i] == -2)
      state[18 + i] = -1;
  }

  return 0;
}

int
count_piece(int *state, int *num_blue, int *num_red)
{
  int i;

  if (state == NULL)
  {
    fprintf(stderr, "count_piece: state is NULL\n");
    return 1;
  }

  if (num_blue != NULL)
  {
    (*num_blue) = 0;
    for (i = 0; i < 9; i++)
    {
      if (state[i] < 0)
        continue;

      (*num_blue)++;
    }
  }

  if (num_red != NULL)
  {
    (*num_red) = 0;
    for (i = 9; i < 18; i++)
    {
      if (state[i] < 0)
        continue;

      (*num_red)++;
    }
  }

  return 0;
}
